@@ -839,3 +839,17 @@ expand_dot_formula <- function(formula, data = NULL) {
   }
   formula
 }
+
+extract_cat_names <- function(x, data) {
+  # extract names of response categories
+  stopifnot(is.brmsformula(x) || is.brmsterms(x))
+  respform <- formula2str(lhs(x$formula))
+  respform <- formula(gsub("\\|+[^~]*~", "~", respform))
+  mr <- model.response(model.frame(respform, data))
+  if (is_ordinal(x) && is.numeric(mr)) {
+    out <- as.character(seq_len(max(mr)))
+  } else {
+    out <- levels(factor(mr))
+  }
+  out
+}
