@@ -6,12 +6,12 @@ test_that("brm produces expected errors", {
   dat <- data.frame(y = rnorm(10), x = rnorm(10), g = rep(1:5, 2))
   
   # formula parsing
-  expect_error(brm(~ x + (1|g), dat), 
+  expect_error(brm(~ x + (1|g), dat, file = "test"), 
                "response variable is missing")
   expect_error(brm(bf(y ~ a, nl = TRUE)),
                "No non-linear parameters specified")
   expect_error(brm(bf(y ~ a, a ~ 1, nl = TRUE), family = acat()),
-               "Non-linear formulas are not yet allowed for this family")
+               "Non-linear formulas are not yet allowed in ordinal models")
   expect_error(brm(bf(y ~ 0 + a), dat, family = cumulative()),
                "Cannot remove the intercept in an ordinal model")
   expect_error(brm(bf(y | se(sei) ~ x, sigma ~ x), dat),
