@@ -69,7 +69,7 @@ test_that(paste("make_standata accepts correct response variables",
                 "depending on the family"), {
   expect_equal(make_standata(y ~ 1, data = data.frame(y = seq(-9.9,0,0.1)), 
                              family = "student")$Y, as.array(seq(-9.9,0,0.1)))
-  expect_equal(make_standata(y ~ 1, data = data.frame(y = 1:10), 
+  expect_equal(make_standata(y | trials(10) ~ 1, data = data.frame(y = 1:10), 
                              family = "binomial")$Y, as.array(1:10))
   expect_equal(make_standata(y ~ 1, data = data.frame(y = 10:20), 
                              family = "poisson")$Y, as.array(10:20))
@@ -128,7 +128,7 @@ test_that(paste("make_standata rejects incorrect response variables",
 })
 
 test_that("make_standata suggests using family bernoulli if appropriate", {
-  expect_message(make_standata(y ~ 1, data = data.frame(y = rep(0:1,5)), 
+  expect_message(make_standata(y | trials(1) ~ 1, data = list(y = rep(0:1,5)), 
                                family = "binomial"),
                  "family 'bernoulli' might be a more efficient choice.")
   expect_message(make_standata(y ~ 1, data = data.frame(y = rep(1:2, 5)), 
@@ -157,8 +157,8 @@ test_that("make_standata returns correct values for addition terms", {
                as.array(c(rep(1:0, 4), 0)))
   expect_equal(make_standata(y | cens(c4, y + 2) ~ 1, data = dat)$rcens, 
                as.array(c(rep(0, 5), dat$y[6:9] + 2)))
-  expect_equal(make_standata(s ~ 1, dat, family = "binomial")$trials, 
-               as.array(rep(9, 9)))
+  sdata <- suppressWarnings(make_standata(s ~ 1, dat, family = "binomial"))
+  expect_equal(sdata$trials, as.array(rep(9, 9)))
   expect_equal(make_standata(s | trials(10) ~ 1, dat, 
                              family = "binomial")$trials, 
                as.array(rep(10, 9)))
@@ -597,6 +597,19 @@ test_that("make_standata allows fixed distributional parameters", {
                "Invalid fixed parameters: 'bias'")
 })
 
+test_that("Cell-mean coding can be disabled", {
+  df <- data.frame(y = 1:10, g = rep(c("a", "b"), 5))
+  bform <- bf(y ~ g) + 
+    lf(disc ~ 0 + g + (0 + g | y), cmc = FALSE) + 
+    cumulative()
+  
+  sdata <- make_standata(bform, df)
+  target <- matrix(rep(0:1, 5), dimnames = list(1:10, "gb"))
+  expect_equal(sdata$X_disc, target)
+  expect_equal(unname(sdata$Z_1_disc_1), as.array(rep(0:1, 5)))
+  expect_true(!"Z_1_disc_2" %in% names(sdata))
+})
+
 test_that("make_standata correctly includes offsets", {
   data <- data.frame(y = rnorm(10), x = rnorm(10), c = 1)
   sdata <- make_standata(bf(y ~ x + offset(c), sigma ~ offset(c + 1)), data)
